'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.twoWayBind = exports.oneWayBind = undefined;

var _flowRuntime = require('flow-runtime');

var _flowRuntime2 = _interopRequireDefault(_flowRuntime);

var _f = require('./f');

var _f2 = _interopRequireDefault(_f);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 *
 * @param target
 */
var buildChangedRequest = function buildChangedRequest(target, pathStack, value) {
	var temp = target;

	while (pathStack.length > 1) {
		var nextProp = pathStack.shift();
		if (temp[nextProp] == null) {
			temp[nextProp] = {};
		}
		temp = temp[nextProp];
	}

	temp[pathStack.shift()] = value;

	return target;
};

/**
 *
 * @param ctx this ref of current component
 */
var doChange = function doChange(ctx, path) {
	return function (value) {
		if (path == null || path.trim() === '') {
			return;
		}

		var propChain = path.split('.');
		var firstProp = propChain[0];
		if (propChain.length === 1) {
			var _next = {};
			_next[firstProp] = value;
			ctx.setState(_next);
			return;
		}

		var next = {};
		next[firstProp] = buildChangedRequest(ctx.state[firstProp], propChain.slice(1), value);
		ctx.setState(next);
	};
};

/**
 * Usage:
 *
 * <code>
 *      import {oneWayBind} from 'react-redux-data-binding';
 *      let $ = oneWayBind(this.props);
 *      <MyComponent $("username") />
 * </code>
 *
 *
 * @param context
 */
var oneWayBind = exports.oneWayBind = function oneWayBind(context) {
	return function (path, defaultValue) {
		return _f2.default.of(context).at(path).value(defaultValue);
	};
};

/**
 * Usage :
 * <code>
 *    import {twoWayBind} from 'react-redux-data-binding';
 *    let $$ = twoWayBind(this);
 *    <MyComponent ...$$("username") />
 * </code>
 *
 * @param context
 */
var twoWayBind = exports.twoWayBind = function twoWayBind(value) {
	for (var _len = arguments.length, event = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
		event[_key - 1] = arguments[_key];
	}

	return function (context, onChange) {
		return function (path, defaultValue) {
			var result = {};
			result[value] = _f2.default.of(context.state).at(path).value(defaultValue);
			event.forEach(function (e) {
				return result[e] = function (value) {
					if (onChange != null) {
						onChange({ path: path, value: value });
						return;
					}
					doChange(context, path)(value);
				};
			});
			return result;
		};
	};
};
//# sourceMappingURL=bind.js.map