'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.twoWayBind = exports.oneWayBind = undefined;

var _flowRuntime = require('flow-runtime');

var _flowRuntime2 = _interopRequireDefault(_flowRuntime);

var _f = require('./f');

var _f2 = _interopRequireDefault(_f);

var _immutable = require('immutable');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 *
 * @param ctx this ref of current component
 */
var doChange = function doChange(ctx, path) {
	return function (value) {
		if (path == null || path.trim() === '') {
			return;
		}

		var propChain = path.split('.');
		var firstProp = propChain[0];
		if (propChain.length === 1) {
			var _next = {};
			_next[firstProp] = value;
			ctx.setState(_next);
			return;
		}

		var next = {};
		next[firstProp] = ctx.state[firstProp].setIn(propChain.slice(1), value);
		ctx.setState(next);
	};
};

/**
 * Usage:
 *
 * <code>
 *      import {oneWayBind} from 'react-redux-data-binding';
 *      let $ = oneWayBind(this.props);
 *      <MyComponent $("username") />
 * </code>
 *
 *
 * @param context
 */
var oneWayBind = exports.oneWayBind = function oneWayBind(context) {
	return function (path, defaultValue) {
		return _f2.default.of((0, _immutable.fromJS)(context)).at(path).value(defaultValue);
	};
};

/**
 * Usage :
 * <code>
 *    import {twoWayBind} from 'react-redux-data-binding';
 *    let $$ = twoWayBind(this);
 *    <MyComponent ...$$("username") />
 * </code>
 *
 * @param context
 */
var twoWayBind = exports.twoWayBind = function twoWayBind(value) {
	for (var _len = arguments.length, event = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
		event[_key - 1] = arguments[_key];
	}

	return function (context, onChange) {
		return function (path, defaultValue) {
			var result = {};
			result[value] = _f2.default.of((0, _immutable.fromJS)(context.state)).at(path).value(defaultValue);
			event.forEach(function (e) {
				return result[event] = function (value) {
					if (onChange != null) {
						onChange({ path: path, value: value });
						return;
					}
					doChange(context, path)(value);
				};
			});
			return result;
		};
	};
};
//# sourceMappingURL=bind.js.map