'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.twoWayBind = exports.oneWayBind = undefined;

var _flowRuntime = require('flow-runtime');

var _flowRuntime2 = _interopRequireDefault(_flowRuntime);

var _f = require('../f');

var _f2 = _interopRequireDefault(_f);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Build the arg of this.setState() for current component
 *
 * @param path
 */
var buildChangedRequest = function buildChangedRequest(path) {
  return function (value) {
    if (path == null || path.trim() === '') {
      return {};
    }

    var result = {};
    var temp = result;
    var stack = path.split('.');

    while (stack.length > 1) {
      temp[stack.shift()] = {};
    }

    temp[stack.shift()] = value;

    return result;
  };
};

/**
 *
 * @param ctx this ref of current component
 */
var doChange = function doChange(ctx) {
  return function (path) {
    return function (value) {
      ctx.setState(buildChangedRequest(path, value));
    };
  };
};

/**
 * Usage:
 *
 * <code>
 *      import {oneWayBind} from 'react-redux-data-binding';
 *      let $ = oneWayBind(this.props);
 *      <MyComponent $("username") />
 * </code>
 *
 *
 * @param context
 */
var oneWayBind = exports.oneWayBind = function oneWayBind(context) {
  return function (path, defaultValue) {
    return _f2.default.of(context).at(path).value(defaultValue);
  };
};

/**
 * Usage :
 * <code>
 * 	import {twoWayBind} from 'react-redux-data-binding';
 * 	let $$ = twoWayBind(this);
 * 	<MyComponent ...$$("username") />
 * </code>
 *
 * @param context
 */
var twoWayBind = exports.twoWayBind = function twoWayBind(context) {
  return function (path, defaultValue) {
    return {
      value: _f2.default.of(context.state).at(path).value(defaultValue),
      onChangeText: function onChangeText(v) {
        return doChange(context, path, v);
      },
      onValueChange: function onValueChange(v) {
        return doChange(context, path, v);
      }
    };
  };
};
//# sourceMappingURL=bind.js.map