'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _flowRuntime = require('flow-runtime');

var _flowRuntime2 = _interopRequireDefault(_flowRuntime);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Optional = function Optional(v) {
	this.v = v;
};

/**
 * Parse the path expr , and iterate the value struct to get the right value .
 * If null found, it returns a new functor with null value.
 *
 * @param path the property path to access the value
 * @returns {Optional}
 */
Optional.prototype.at = function (path) {
	if (this.v == null || path == null || path.trim() === '') {
		return new Optional(this.v);
	}

	var result = this.v;
	var stack = path.trim().split('.');

	while (stack.length) {
		result = result[stack.shift()];
		if (result == undefined) {
			return new Optional(null);
		}
	}

	return new Optional(result);
};

/**
 * Map the value to another , and return a new functor with the result.
 * if the mapped result is instance of Optional , the value of it will be taken as the functor's input.
 *
 * @param lambda
 * @returns {Optional}
 */
Optional.prototype.map = function (lambda) {
	var v = this.v != null ? lambda(this.v) : null;
	return new Optional(v instanceof Optional ? v.v : v);
};

Optional.prototype.peek = function (lambda) {
	lambda(this.v);
	return new Optional(this.v);
};

/**
 * get the value of functor, if the value is null and the default value is supplied, it will return the default value as result
 *
 * if the defaultValue is instanceof function , whatever the value is null or not , call the function and return the result.
 *
 *
 * @param dv default value (should be a plain value or a function)
 * @returns {any}
 */
Optional.prototype.value = function (dv) {
	if (dv != null && typeof dv === 'function') {
		return dv(this.v);
	}

	return this.v == null ? dv : this.v;
};

exports.default = Optional;
module.exports = exports['default'];
//# sourceMappingURL=optional.js.map